; $Id: dave_mdicer.pro,v 1.4 2002/07/18 20:30:22 john Exp $
; Written by J.R.D. Copley
;************************************************************************************************
pro show_ptrs,message
;************************************************************************************************
;
compile_opt strictarr
;
; Prints number of pointers.
result=ptr_valid(count=n)
if (n_elements(message) eq 0) then begin
	print,'There are ',n,' valid pointers.'
endif else begin
	print,'At ',message,',',n,' valid pointers.'
endelse
end


;************************************************************************************************
pro dave_mdicerQuit,event
;************************************************************************************************
;
compile_opt strictarr
;
widget_control,event.top,/destroy
return
end


;************************************************************************************************
pro dave_mdicerCleanup,tlb
;************************************************************************************************
; Cleanup routine.
;
compile_opt strictarr
;
;show_ptrs,'start of cleanup'
widget_control,tlb,get_uvalue = pState,/no_copy
s = size((*pState).notifyIDs)
if s[0] eq 1 then count = 0 else count = s[2]-1
for j = 0,count do begin
  mdcInfo = {mdcEvent,$
                  ID:(*pState).notifyIDs[0,j],$
                  Top:(*pState).notifyIDs[1,j],$
                  Handler:0l,$
                  inPtr:ptr_new(*(*pState).inPtr)}
   if widget_info((*pState).notifyIDs[0,j],/valid_id) then begin $
      widget_control,(*pState).notifyIDs[0,j],send_event = mdcInfo
   endif
endfor
if (ptr_valid(pstate)) then begin
	if (ptr_valid((*pstate).pqty)) then ptr_free,(*pstate).pqty
	if (ptr_valid((*pstate).perr)) then ptr_free,(*pstate).perr
	if (ptr_valid((*pstate).px)) then ptr_free,(*pstate).px
	if (ptr_valid((*pstate).py)) then ptr_free,(*pstate).py
	if (ptr_valid((*pstate).activePtr)) then ptr_free,(*pstate).activePtr
	if (ptr_valid((*pstate).filesPtr)) then ptr_free,(*pstate).filesPtr
	if (ptr_valid((*pstate).winstate.xPtr)) then ptr_free,(*pstate).winstate.xPtr
	if (ptr_valid((*pstate).winstate.yPtr)) then ptr_free,(*pstate).winstate.yPtr
	ptr_free,pstate
endif
;show_ptrs,'end of cleanup'
return
end


;************************************************************************************************
pro dave_mdicer_populate,filename,pState=pState,davePtr=davePtr
;************************************************************************************************
; This routine populates portions of the structure (*pstate).
compile_opt strictarr
;
; If this is the first dataset, populate all relevant fields.
if ((*pstate).ndatasets eq 1) then begin
; If the x array is 2-dimensional, set the number of datasets = 0 and return.
	ndx=size(((*(*(*davePtr).dataStrPtr).commonStr.histPtr).x),/n_dimensions)
	if (ndx gt 1) then begin
		res=dialog_message('The x array must be 1-dimensional.',/error)
		(*pstate).ndatasets=0
		return
	endif
; If the y array is 2-dimensional, set the number of datasets = 0 and return.
	ndy=size(((*(*(*davePtr).dataStrPtr).commonStr.histPtr).y),/n_dimensions)
	if (ndy gt 1) then begin
		res=dialog_message('The y array must be 1-dimensional.',/error)
		(*pstate).ndatasets=0
		return
	endif
	*(*pstate).pqty=((*(*(*davePtr).dataStrPtr).commonStr.histPtr).qty)
	*(*pstate).perr=((*(*(*davePtr).dataStrPtr).commonStr.histPtr).err)
	*(*pstate).px=((*(*(*davePtr).dataStrPtr).commonStr.histPtr).x)
	*(*pstate).py=((*(*(*davePtr).dataStrPtr).commonStr.histPtr).y)
	(*pstate).xlabel=(*(*davePtr).dataStrPtr).commonStr.xlabel
	(*pstate).ylabel=(*(*davePtr).dataStrPtr).commonStr.ylabel
	(*pstate).histlabel=(*(*davePtr).dataStrPtr).commonStr.histlabel
	(*pstate).xunits=(*(*davePtr).dataStrPtr).commonStr.xunits
	(*pstate).yunits=(*(*davePtr).dataStrPtr).commonStr.yunits
	(*pstate).histunits=(*(*davePtr).dataStrPtr).commonStr.histunits
	(*pstate).xtype=(*(*davePtr).dataStrPtr).commonStr.xtype
	(*pstate).ytype=(*(*davePtr).dataStrPtr).commonStr.ytype
	*(*pstate).activePtr=1
	*(*pstate).filesPtr=filename
;
; If what's to be plotted is histogrammed in x, convert it to points format.
	if ((*pstate).xtype eq "histogram") then begin
		npx=0.5*((*(*pstate).px)[0:n_elements(*(*pstate).px)-2]+$
			(*(*pstate).px)[1:n_elements(*(*pstate).px)-1])
		ptr_free,(*pstate).px
		(*pstate).px=ptr_new(npx)
	endif
;
; If it's histogrammed in y, convert it to points format.
	if ((*pstate).ytype eq "histogram") then begin
		npy=0.5*((*(*pstate).py)[0:n_elements(*(*pstate).py)-2]+$
			(*(*pstate).py)[1:n_elements(*(*pstate).py)-1])
		ptr_free,(*pstate).py
		(*pstate).py=ptr_new(npy)
	endif
;
	return
endif
;
; If this is not the first dataset, populate the qty, err, active and file fields,
;	but first do some compatibility tests.
; If the dimensions of qty do not match, decrement the number of datasets and return.
dims=size((*(*(*davePtr).dataStrPtr).commonStr.histPtr).qty,/dimensions)
if ((dims[0] ne (*pstate).xdim) or (dims[1] ne (*pstate).ydim)) then begin
	res=dialog_message('Dataset dimensions do not match.',/error)
	(*pstate).ndatasets=(*pstate).ndatasets-1
	return
endif
; If the x arrays do not match, decrement the number of datasets and return.
; If necessary, convert to points format before making the comparison.
if ((*pstate).xtype eq "histogram") then begin
	npx=0.5*((*(*pstate).px)[0:n_elements(*(*pstate).px)-2]+$
		(*(*pstate).px)[1:n_elements(*(*pstate).px)-1])
	ptr_free,(*pstate).px
	(*pstate).px=ptr_new(npx)
endif
nmatch=total((*(*(*davePtr).dataStrPtr).commonStr.histPtr).x eq *(*pstate).px)
if (nmatch ne dims[0]) then begin
	res=dialog_message('x values do not match.',/error)
	(*pstate).ndatasets=(*pstate).ndatasets-1
	return
endif
; If the y arrays do not match, decrement the number of datasets and return.
; If necessary, convert to points format before making the comparison.
if ((*pstate).ytype eq "histogram") then begin
	npy=0.5*((*(*pstate).py)[0:n_elements(*(*pstate).py)-2]+$
		(*(*pstate).py)[1:n_elements(*(*pstate).py)-1])
	ptr_free,(*pstate).py
	(*pstate).py=ptr_new(npy)
endif
nmatch=total((*(*(*davePtr).dataStrPtr).commonStr.histPtr).y eq *(*pstate).py)
if (nmatch ne dims[1]) then begin
	res=dialog_message('y values do not match.',/error)
	(*pstate).ndatasets=(*pstate).ndatasets-1
	return
endif
;
; All is well, so append to the appropriate arrays.
oldptrs=[(*pstate).pqty,$
	(*pstate).perr,$
	(*pstate).activePtr,$
	(*pstate).filesPtr]
(*pstate).pqty=ptr_new([[[*(*pstate).pqty]],[[(*(*(*davePtr).dataStrPtr).commonStr.histPtr).qty]]],/no_copy)
(*pstate).perr=ptr_new([[[*(*pstate).perr]],[[(*(*(*davePtr).dataStrPtr).commonStr.histPtr).err]]],/no_copy)
(*pstate).activePtr=ptr_new([*(*pstate).activePtr,1])
(*pstate).filesPtr=ptr_new([*(*pstate).filesPtr,filename])
ptr_free,oldptrs
;
end


;************************************************************************************************
pro dave_mdicerRead,event
;************************************************************************************************
; This routine reads a DAVE file or a DCS raw data file.
;
compile_opt strictarr
;
widget_control,event.top,get_uvalue = pState,/no_copy
;
readDAVE=widget_info(event.top,find_by_uname="readDAVE")
readDCS=widget_info(event.top,find_by_uname="readDCS")
;
; Read a DAVE file.
if (event.id eq readDAVE) then begin
	widget_control,readDCS,sensitive=0
	filename=dialog_pickfile(filter="*.dave",path=(*!dave_defaults).workdir)
	if (filename ne "") then restore,filename
endif
;
; Read a DCS raw data file.
if (event.id eq readDCS) then begin
	widget_control,readDAVE,sensitive=0
	widget_control,(*pstate).rbutton,sensitive=1
	filename=dialog_pickfile(filter="*.dcs*",path=(*!dave_defaults).datdir)
	if (filename ne "") then begin
		dave_makeDavePtr,davePtr=davePtr
		dcs_definedavestructure,davePtr=davePtr
		dcs_read_binaryOctavefile,filename,0,0,variables_read,davePtr=davePtr
	endif
endif
;
; If there is no file (e.g. "Cancel" button is hit), and no files
; have yet been read destroy the widget. Otherwise return.
if (filename eq "") then begin
	if ((*pstate).ndatasets eq 0) then begin
		widget_control,event.top,set_uvalue = pState,/no_copy
		dave_mdicerQuit,event
	endif else widget_control,event.top,set_uvalue = pState,/no_copy
	return
endif
;
(*pstate).readingDCS=(event.id eq readDCS)
;
if ((*pstate).readingDCS and (*pstate).ndatasets eq 0) then begin
; First time around, define some additional quantities that may be needed.
	*(*pstate).tchansPtr=$
		(*(*(*davePtr).dataStrPtr).specificPtr).tchanlook[0:999]*0.05
	*(*pstate).timesPtr=$
		(*(*(*(*davePtr).dataStrPtr).specificPtr).time_propsPtr)[0:999]
	*(*pstate).energiesPtr=$
		(*(*(*(*davePtr).dataStrPtr).specificPtr).energy_propsPtr)[0:999]
	elastic_channel=$
		where(abs(*(*pstate).energiesPtr) eq min(abs(*(*pstate).energiesPtr)))
	(*pstate).elastic_time=(*(*pstate).timesPtr)[elastic_channel]
endif
;
; Having read in a file, sensitize most of the widgets.
widget_control,(*pstate).wprint,/sensitive
widget_control,(*pstate).wsplot,/sensitive
widget_control,(*pstate).wsdata,/sensitive
widget_control,(*pstate).xymenu,/sensitive
widget_control,(*pstate).gpmenu,/sensitive
widget_control,(*pstate).optmenu,/sensitive
widget_control,(*pstate).setmenu,/sensitive
widget_control,(*pstate).lpanbut,/sensitive
widget_control,(*pstate).rpanbut,/sensitive
widget_control,(*pstate).dpanbut,/sensitive
widget_control,(*pstate).upanbut,/sensitive
widget_control,(*pstate).zoinbut,/sensitive
widget_control,(*pstate).zootbut,/sensitive
widget_control,(*pstate).xyrbnbut,/sensitive
;
(*pstate).ndatasets=(*pstate).ndatasets+1
ndatasets=(*pstate).ndatasets
dave_mdicer_populate,filename,pState=pState,davePtr=davePtr
free_all_ptrs,davePtr
;
; If number of datasets has been decremented (e.g. because
; dimensions do not match), return.
if ((*pstate).ndatasets lt ndatasets) then begin
	widget_control,event.top,set_uvalue = pState,/no_copy
	return
endif
;
dims=size(*(*pstate).pqty,/dimensions)
;
ndatasets=(*pstate).ndatasets
;
; Populate the file information box.
n=n_elements(*(*pstate).filesPtr)
information=strcompress(string(indgen(n)+1))+"  "+*(*pstate).filesPtr
widget_control,(*pstate).info,set_value=information
;
; Fill in information for "Abscissa" menu items.
widget_control,(*pstate).xbutton,set_value='['+(*pstate).xlabel+']'
widget_control,(*pstate).ybutton,set_value=(*pstate).ylabel
;
; Use x and y dimensions to set maximum slider values.
plotx=(*pstate).plot_vs_x
widget_control,(*pstate).xyvalbut,set_slider_max=dims[plotx]
widget_control,(*pstate).xyrngbut,set_slider_max=dims[plotx]/2
widget_control,(*pstate).xyrbnbut,set_slider_max=dims[1-plotx]/10 > 6
widget_control,(*pstate).grps,set_slider_max=dims[plotx]
;
(*pstate).xdim=dims[0]
(*pstate).ydim=dims[1]
;
; Update the plot.
wset,(*pstate).winstate.winPix
widget_control,event.top,set_uvalue = pState,/no_copy
dave_mdicerPlot,event
widget_control,event.top,get_uvalue = pState,/no_copy
wset,(*pstate).winstate.winVis
device,copy = [0,0,!d.x_size,!d.y_size,0,0,(*pstate).winstate.winPix]
wshow,(*pstate).winstate.winVis
widget_control,event.top,set_uvalue = pState,/no_copy
;
return
end


;************************************************************************************************
pro dave_mdicerSaveplot,event
;************************************************************************************************
;
compile_opt strictarr
;
filename=dialog_pickfile(title="Enter saved plot file name.",filter="*.plt")
if (filename eq "") then return
if (strpos(filename,".plt") eq -1) then filename=filename+".plt"
;
widget_control,event.top,get_uvalue=pstate,/no_copy
save,filename=filename,pstate
;
widget_control,event.top,set_uvalue=pstate,/no_copy
;
end


;************************************************************************************************
pro dave_mdicerRestoreplot,event
;************************************************************************************************
;
compile_opt strictarr
;
widget_control,event.top,get_uvalue=pstate,/no_copy
;
; First of all store the current values of the pixmap and screen windows,
; the draw window, and all the various widgets created in the widget
; definition module.
winPix=(*pstate).winstate.winPix
winVis=(*pstate).winstate.winVis
win=(*pstate).winstate.win
wprint=(*pstate).wprint
wsplot=(*pstate).wsplot
wrplot=(*pstate).wrplot
wsdata=(*pstate).wsdata
xymenu=(*pstate).xymenu
gpmenu=(*pstate).gpmenu
optmenu=(*pstate).optmenu
setmenu=(*pstate).setmenu
xbutton=(*pstate).xbutton
ybutton=(*pstate).ybutton
sgrpbut=(*pstate).sgrpbut
mgrpbut=(*pstate).mgrpbut
agrpbut=(*pstate).agrpbut
ebutton=(*pstate).ebutton
pbutton=(*pstate).pbutton
rbutton=(*pstate).rbutton
tthbutt=(*pstate).tthbutt
athbutt=(*pstate).athbutt
itmbutt=(*pstate).itmbutt
ienbutt=(*pstate).ienbutt
it4butt=(*pstate).it4butt
revbutt=(*pstate).revbutt
abutton=(*pstate).abutton
tbutton=(*pstate).tbutton
xyvalbut=(*pstate).xyvalbut
xyrngbut=(*pstate).xyrngbut
xyrbnbut=(*pstate).xyrbnbut
lpanbut=(*pstate).lpanbut
rpanbut=(*pstate).rpanbut
dpanbut=(*pstate).dpanbut
upanbut=(*pstate).upanbut
zoinbut=(*pstate).zoinbut
zootbut=(*pstate).zootbut
zfrcbut=(*pstate).zfrcbut
pfrcbut=(*pstate).pfrcbut
gbase=(*pstate).gbase
grps=(*pstate).grps
info=(*pstate).info
cpos=(*pstate).cpos
xlval=(*pstate).xlval
xrval=(*pstate).xrval
ydval=(*pstate).ydval
yuval=(*pstate).yuval
;
; Now restore the plot.
filename=dialog_pickfile(title="Enter name of plot file to be restored.",filter="*.plt")
if (filename eq "") then return
restore,filename=filename
;
; Substitute the current values back into pstate.
(*pstate).winstate.winPix=winPix
(*pstate).winstate.winVis=winVis
(*pstate).winstate.win=win
(*pstate).wprint=wprint
(*pstate).wsplot=wsplot
(*pstate).wrplot=wrplot
(*pstate).wsdata=wsdata
(*pstate).xymenu=xymenu
(*pstate).gpmenu=gpmenu
(*pstate).optmenu=optmenu
(*pstate).setmenu=setmenu
(*pstate).xbutton=xbutton
(*pstate).ybutton=ybutton
(*pstate).sgrpbut=sgrpbut
(*pstate).mgrpbut=mgrpbut
(*pstate).agrpbut=agrpbut
(*pstate).ebutton=ebutton
(*pstate).pbutton=pbutton
(*pstate).rbutton=rbutton
(*pstate).tthbutt=tthbutt
(*pstate).athbutt=athbutt
(*pstate).itmbutt=itmbutt
(*pstate).ienbutt=ienbutt
(*pstate).it4butt=it4butt
(*pstate).revbutt=revbutt
(*pstate).abutton=abutton
(*pstate).tbutton=tbutton
(*pstate).xyvalbut=xyvalbut
(*pstate).xyrngbut=xyrngbut
(*pstate).xyrbnbut=xyrbnbut
(*pstate).lpanbut=lpanbut
(*pstate).rpanbut=rpanbut
(*pstate).dpanbut=dpanbut
(*pstate).upanbut=upanbut
(*pstate).zoinbut=zoinbut
(*pstate).zootbut=zootbut
(*pstate).zfrcbut=zfrcbut
(*pstate).pfrcbut=pfrcbut
(*pstate).gbase=gbase
(*pstate).grps=grps
(*pstate).info=info
(*pstate).cpos=cpos
(*pstate).xlval=xlval
(*pstate).xrval=xrval
(*pstate).ydval=ydval
(*pstate).yuval=yuval
;
; Sensitize the menu widgets.
readDAVE=widget_info(event.top,find_by_uname="readDAVE")
readDCS=widget_info(event.top,find_by_uname="readDCS")
widget_control,readDCS,sensitive=(*pstate).readingDCS
widget_control,readDAVE,sensitive=not (*pstate).readingDCS
widget_control,(*pstate).rbutton,sensitive=(*pstate).readingDCS
widget_control,(*pstate).wprint,/sensitive
widget_control,(*pstate).wsplot,/sensitive
widget_control,(*pstate).wsdata,/sensitive
widget_control,(*pstate).xymenu,/sensitive
widget_control,(*pstate).gpmenu,/sensitive
widget_control,(*pstate).optmenu,/sensitive
widget_control,(*pstate).setmenu,/sensitive
;
;	Sensitize the pan and zoom buttons.
widget_control,(*pstate).lpanbut,/sensitive
widget_control,(*pstate).rpanbut,/sensitive
widget_control,(*pstate).dpanbut,/sensitive
widget_control,(*pstate).upanbut,/sensitive
widget_control,(*pstate).zoinbut,/sensitive
widget_control,(*pstate).zootbut,/sensitive
;
; Sensitize the rebin button.
widget_control,(*pstate).xyrbnbut,/sensitive
;
; Set sensitivity of "Number of groups" button.
if ((*pstate).one) then widget_control,(*pstate).gbase,sensitive=0
if ((*pstate).some) then widget_control,(*pstate).gbase,sensitive=1
if ((*pstate).all) then widget_control,(*pstate).gbase,sensitive=0
;
; Define slider maximum values and current values.
dims=[(*pstate).xdim,(*pstate).ydim]
plotx=(*pstate).plot_vs_x
widget_control,(*pstate).grps,set_slider_max=dims[plotx]
widget_control,(*pstate).grps,set_value=(*pstate).ngrps
;
widget_control,(*pstate).pfrcbut,set_value=(*pstate).pfrac
widget_control,(*pstate).zfrcbut,set_value=(*pstate).zfrac
;
widget_control,(*pstate).xyvalbut,set_slider_max=dims[plotx]
widget_control,(*pstate).xyrngbut,set_slider_max=dims[plotx]/2
widget_control,(*pstate).xyrbnbut,set_slider_max=dims[1-plotx]/10 > 6
if (plotx) then begin
	widget_control,(*pstate).xyvalbut,set_value=(*pstate).yvalue
	widget_control,(*pstate).xyrngbut,set_value=(*pstate).ywidth
	widget_control,(*pstate).xyrbnbut,set_value=(*pstate).xrebin
endif else begin
	widget_control,(*pstate).xyvalbut,set_value=(*pstate).xvalue
	widget_control,(*pstate).xyrngbut,set_value=(*pstate).xwidth
	widget_control,(*pstate).xyrbnbut,set_value=(*pstate).yrebin
endelse
;
; Populate the file information box.
n=n_elements(*(*pstate).filesPtr)
information=strcompress(string(indgen(n)+1))+"  "+*(*pstate).filesPtr
widget_control,(*pstate).info,set_value=information
;
; Update labels on certain menu items.
if (plotx) then begin
	widget_control,(*pstate).xbutton,set_value='['+(*pstate).xlabel+']'
	widget_control,(*pstate).ybutton,set_value=(*pstate).ylabel
endif else begin
	widget_control,(*pstate).xbutton,set_value=(*pstate).xlabel
	widget_control,(*pstate).ybutton,set_value='['+(*pstate).ylabel+']'
endelse
;
if ((*pstate).one) then begin
	widget_control,(*pstate).sgrpbut,set_value='[Single group]'
	widget_control,(*pstate).mgrpbut,set_value='Several groups'
	widget_control,(*pstate).agrpbut,set_value='All in one group'
endif
if ((*pstate).some) then begin
	widget_control,(*pstate).sgrpbut,set_value='Single group'
	widget_control,(*pstate).mgrpbut,set_value='[Several groups]'
	widget_control,(*pstate).agrpbut,set_value='All in one group'
endif
if ((*pstate).all) then begin
	widget_control,(*pstate).sgrpbut,set_value='Single group'
	widget_control,(*pstate).mgrpbut,set_value='Several groups'
	widget_control,(*pstate).agrpbut,set_value='[All in one group]'
endif
;
if ((*pstate).plot_eb) then $
	widget_control,(*pstate).ebutton,set_value="Hide error bars" $
else widget_control,(*pstate).ebutton,set_value="Show error bars"
;
if ((*pstate).join_pts eq 1) then $
	widget_control,(*pstate).pbutton,set_value="Do not join points"
if ((*pstate).join_pts eq -1) then $
	widget_control,(*pstate).pbutton,set_value="Join points"
;
; Update the plot.
wset,(*pstate).winstate.winPix
widget_control,event.top,set_uvalue=pstate,/no_copy
dave_mdicerPlot,event
widget_control,event.top,get_uvalue=pstate,/no_copy
wset,(*pstate).winstate.winVis
device,copy = [0,0,!d.x_size,!d.y_size,0,0,(*pstate).winstate.winPix]
wshow,(*pstate).winstate.winVis
widget_control,event.top,set_uvalue=pstate,/no_copy
;
end


;************************************************************************************************
pro dave_mdicerClear,event
;************************************************************************************************
;
compile_opt strictarr
;
widget_control,event.top,get_uvalue=pstate,/no_copy
;
; Sensitize the menu widgets.
readDAVE=widget_info(event.top,find_by_uname="readDAVE")
readDCS=widget_info(event.top,find_by_uname="readDCS")
widget_control,readDCS,sensitive=1
widget_control,readDAVE,sensitive=1
widget_control,(*pstate).rbutton,sensitive=0
widget_control,(*pstate).wprint,sensitive=0
widget_control,(*pstate).wsplot,sensitive=0
widget_control,(*pstate).wsdata,sensitive=0
widget_control,(*pstate).xymenu,sensitive=0
widget_control,(*pstate).gpmenu,sensitive=0
widget_control,(*pstate).optmenu,sensitive=0
widget_control,(*pstate).setmenu,sensitive=0
;
;	Sensitize the pan and zoom buttons.
widget_control,(*pstate).lpanbut,sensitive=0
widget_control,(*pstate).rpanbut,sensitive=0
widget_control,(*pstate).dpanbut,sensitive=0
widget_control,(*pstate).upanbut,sensitive=0
widget_control,(*pstate).zoinbut,sensitive=0
widget_control,(*pstate).zootbut,sensitive=0
;
; Sensitize the rebin button.
widget_control,(*pstate).xyrbnbut,sensitive=1
;
; Define slider maximum values and current values.
widget_control,(*pstate).grps,set_slider_max=100
widget_control,(*pstate).grps,set_value=1
;
widget_control,(*pstate).pfrcbut,set_value=20
widget_control,(*pstate).zfrcbut,set_value=80
;
widget_control,(*pstate).xyvalbut,set_slider_max=100
widget_control,(*pstate).xyrngbut,set_slider_max=100
widget_control,(*pstate).xyrbnbut,set_slider_max=100
widget_control,(*pstate).xyvalbut,set_value=1
widget_control,(*pstate).xyrngbut,set_value=0
widget_control,(*pstate).xyrbnbut,set_value=1
;
; Clear the file information box.
widget_control,(*pstate).info,set_value=""
;
; Update labels on certain menu items.
widget_control,(*pstate).xbutton,set_value='x: (not yet defined)'
widget_control,(*pstate).ybutton,set_value='y: (not yet defined)'
;
widget_control,(*pstate).sgrpbut,set_value='Single group'
widget_control,(*pstate).mgrpbut,set_value='Several groups'
widget_control,(*pstate).agrpbut,set_value='[All in one group]'
;
widget_control,(*pstate).ebutton,set_value="Show error bars"
;
widget_control,(*pstate).pbutton,set_value="Join points"
;
(*pstate).ngrps=1
(*pstate).ndatasets=0
(*pstate).xvalue=1
(*pstate).yvalue=1
(*pstate).xwidth=0
(*pstate).ywidth=0
(*pstate).xrebin=1
(*pstate).yrebin=1
(*pstate).plot_vs_x=1
(*pstate).plot_vs_y=0
(*pstate).plot_eb=0
(*pstate).join_pts=-1
(*pstate).plotv2th=0
(*pstate).plota2th=0
(*pstate).plotvst=0
(*pstate).plotvsE=0
(*pstate).plott4E=0
(*pstate).one=0
(*pstate).some=0
(*pstate).all=1
(*pstate).pan=0
(*pstate).pfrac=20
(*pstate).zoom=0
(*pstate).zfrac=80
(*pstate).xdim=1
(*pstate).ydim=1
;
wset,(*pstate).winstate.winPix
xsize=!d.x_size
ysize=!d.y_size
wdelete,(*pstate).winstate.winPix
window,/free,/pixmap,xsize=xsize,ysize=ysize
winPix=!d.window
(*pstate).winstate.winPix=winPix
wset,(*pstate).winstate.winVis
device,copy = [0,0,!d.x_size,!d.y_size,0,0,(*pstate).winstate.winPix]
wshow,(*pstate).winstate.winVis
;
widget_control,event.top,set_uvalue=pstate,/no_copy
;
end


;************************************************************************************************
pro dave_mdicerPlot,event
;************************************************************************************************
;
compile_opt strictarr
;
widget_control,event.top,get_uvalue=pstate,/no_copy
;
; Define some variables, depending whether the abscissa is x or y.
plot_vs_x=(*pstate).plot_vs_x
;
samedim =   (plot_vs_x) ? (*pstate).xdim   : (*pstate).ydim
samerebin = (plot_vs_x) ? (*pstate).xrebin : (*pstate).yrebin
samelabel = (plot_vs_x) ? (*pstate).xlabel : (*pstate).ylabel
;
othrvalue = (plot_vs_x) ? (*pstate).yvalue : (*pstate).xvalue
othrwidth = (plot_vs_x) ? (*pstate).ywidth : (*pstate).xwidth
othrdim =   (plot_vs_x) ? (*pstate).ydim   : (*pstate).xdim
othrlabel = (plot_vs_x) ? (*pstate).ylabel : (*pstate).xlabel
;
abscis =    (plot_vs_x) ? *(*pstate).px    : *(*pstate).py
tot_index = (plot_vs_x) ? 2                : 1
;
; If plotting vs 2theta or abs(2theta), redefine the abscissa.
if ((*pstate).plotv2th) then begin
	dcs_getangles,nhedet=913,angles
	abscis=angles
	samelabel="Scattering angle"
endif
if ((*pstate).plota2th) then begin
	dcs_getangles,nhedet=913,angles
	abscis=abs(angles)
	samelabel="ABS(Scattering angle)"
endif
;
; If plotting vs time or energy, redefine the abscissa.
if ((*pstate).plotvst) then begin
	abscis=*(*pstate).timesPtr
	samelabel="Time sample-detector (us)"
endif
if ((*pstate).plotvsE or (*pstate).plott4E) then begin
	abscis=*(*pstate).energiesPtr
	samelabel="Energy transfer (meV)"
endif
;
; Define ordinate and its error when plotting sum over other dimension.
if ((*pstate).all) then begin
	ordina=total((*(*pstate).pqty),tot_index)/othrdim
	oerror=sqrt(total((*(*pstate).perr)^2,tot_index))/othrdim
	title=(*pstate).histlabel+": sum over all "+othrlabel
endif
;
; Define ordinate and its error when plotting several "groups".
; This is only allowed if there is only one dataset.
if ((*pstate).some) then begin
	if ((*pstate).ndatasets ne 1) then begin
		res=dialog_message("Multiple groups cannot be plotted if there are multiple files.")
		widget_control,event.top,set_uvalue=pstate,/no_copy
		return
	endif else begin
		ndim=othrdim/(*pstate).ngrps
		ordina=fltarr(samedim,(*pstate).ngrps)
		oerror=fltarr(samedim,(*pstate).ngrps)
; Loop over groups, generating lower and upper values of the third dimension.
		kc=othrdim-ndim*(*pstate).ngrps
		ke=-1
		for k=0,(*pstate).ngrps-1 do begin
			if (k lt kc) then begin
				ks=ke+1
				ke=ks+ndim
			endif else begin
				ks=ke+1
				ke=ks+ndim-1
			endelse
; Generate the ordinate and its error.
;	Treat separately the case where there is only one contribution to the group.
			if (ks eq ke) then begin
				if ((*pstate).plot_vs_x) then begin
					ordina[*,k]=(*(*pstate).pqty)[*,ks]
					oerror[*,k]=sqrt((*(*pstate).perr)[*,ks]^2)
				endif else begin
					ordina[*,k]=(*(*pstate).pqty)[ks,*]
					oerror[*,k]=sqrt((*(*pstate).perr)[ks,*]^2)
				endelse
			endif else begin
				if ((*pstate).plot_vs_x) then begin
					ordina[*,k]=total((*(*pstate).pqty)[*,ks:ke],2)/(ke-ks+1);;
					oerror[*,k]=sqrt(total((*(*pstate).perr)[*,ks:ke]^2,2))/(ke-ks+1);;
				endif else begin
					ordina[*,k]=total((*(*pstate).pqty)[ks:ke,*],1)/(ke-ks+1)
					oerror[*,k]=sqrt(total((*(*pstate).perr)[ks:ke,*]^2,1))/(ke-ks+1)
				endelse
			endelse
		endfor
		title=(*pstate).histlabel+": "+othrlabel+"-"+$
			strcompress((*pstate).ngrps)+" groups"
	endelse
endif
;
; Define ordinate and its error when plotting a single "group".
if ((*pstate).one) then begin
	othrval1=othrvalue-othrwidth > 1
	othrval2=othrvalue+othrwidth < othrdim
	if (othrval1 eq othrval2) then begin
		if ((*pstate).plot_vs_x) then begin
			ordina=(*(*pstate).pqty)[*,othrvalue-1,*]
			oerror=(*(*pstate).perr)[*,othrvalue-1,*]
		endif else begin
			ordina=(*(*pstate).pqty)[othrvalue-1,*,*]
			oerror=(*(*pstate).perr)[othrvalue-1,*,*]
		endelse
		title=(*pstate).histlabel+": "+othrlabel+" ="+strcompress(othrvalue)
	endif else begin
		if ((*pstate).plot_vs_x) then begin
			ordina=total((*(*pstate).pqty)[*,othrval1-1:othrval2-1,*],2)$
				/(othrval2-othrval1+1)
			oerror=sqrt(total((*(*pstate).perr)[*,othrval1-1:othrval2-1,*]^2,2))$
				/(othrval2-othrval1+1)
		endif else begin
			ordina=total((*(*pstate).pqty)[othrval1-1:othrval2-1,*,*],1)$
				/(othrval2-othrval1+1)
			oerror=sqrt(total((*(*pstate).perr)[othrval1-1:othrval2-1,*,*]^2,1))$
				/(othrval2-othrval1+1)
		endelse
		title=(*pstate).histlabel+": "+othrlabel+$
			" ="+strcompress(othrval1)+" to"+strcompress(othrval2)
	endelse
endif
;
;	If required, rebin the data.
; Presently this is only allowed if there is only one dataset.
if (samerebin gt 1) then begin
	if ((*pstate).ndatasets eq 1) then begin
		res=dialog_message("This feature is not yet implemented.")
		(*pstate).xrebin=1
		(*pstate).yrebin=1
		widget_control,(*pstate).xyrbnbut,set_value=1
		widget_control,event.top,set_uvalue=pstate,/no_copy
		return
		newdim=samedim/samerebin
		if (samedim gt samerebin*newdim) then $
			abscis=abscis[0:samerebin*newdim-1]
		abscis=total(reform(abscis,samerebin,newdim),1)/samerebin
		nd=size(ordina,/n_dimensions)
		if (nd eq 1) then begin
			ordina=reform(ordina)
			ordina=congrid(ordina,newdim)
			oerror=ordina*0.5
		endif else begin
			ordina=congrid(ordina,newdim,(*pstate).ngrps)
			oerror=ordina*0.5
		endelse
	endif else begin
		res=dialog_message("This feature is not yet implemented.")
		(*pstate).xrebin=1
		(*pstate).yrebin=1
		widget_control,(*pstate).xyrbnbut,set_value=1
		widget_control,event.top,set_uvalue=pstate,/no_copy
		return
	endelse
endif
;
; If plotting raw DCS data in certain ways, involving transformation of the ordinate,
;	construct a variable (*factorPtr) with the same length as the (modified) ordinate.
; The dimension of *unityPtr is the product of all but the first dimension of ordina.
if ((*pstate).plotvst or (*pstate).plotvsE) then begin
	dims=size(ordina,/dimensions)
	unityPtr=ptr_new(bytarr(n_elements(ordina)/dims[0])+1)
	factorPtr=ptr_new((*(*pstate).tchansPtr)#(*unityPtr))
	ordina=ordina[0:999,*,*]/(*factorPtr)
	oerror=oerror[0:999,*,*]/(*factorPtr)
	ptr_free,unityPtr
	ptr_free,factorPtr
endif
if ((*pstate).plott4E) then begin
	dims=size(ordina,/dimensions)
	unityPtr=ptr_new(bytarr(n_elements(ordina)/dims[0])+1)
	factorPtr=ptr_new((((*(*pstate).timesPtr)/(*pstate).elastic_time)^4/(*(*pstate).tchansPtr))#(*unityPtr))
	ordina=ordina[0:999,*,*]*(*factorPtr)
	oerror=oerror[0:999,*,*]*(*factorPtr)
endif
;
if ((*pstate).savedata) then begin
	filename=dialog_pickfile(title="Enter saved data file name.",filter="*.dat")
	if (filename eq "") then begin
		widget_control,event.top,set_uvalue=pstate,/no_copy
		return
	endif
	if (strpos(filename,".dat") eq -1) then filename=filename+".dat"
	openw,unit,filename,/get_lun
	nvals=size(abscis,/dimension)
	for k=0,nvals[0]-1 do begin
		printf,unit,abscis[k],ordina[k,*],oerror[k,*]
	endfor
	free_lun,unit
endif
;
;	If required, perform autoscaling.
if ((*pstate).winstate.autoscale eq 1) then begin
	minabs=min(abscis)
	maxabs=max(abscis)
	absrange=maxabs-minabs
  (*pstate).winstate.xrange = [minabs-absrange*0.05,maxabs+absrange*0.05]
	minord=min(ordina)
	maxord=max(ordina)
	ordrange=maxord-minord
  (*pstate).winstate.yrange = [minord-ordrange*0.05,maxord+ordrange*0.05]
endif
;
; If required, perform panning operation.
if ((*pstate).pan ne 0) then (*pstate).winstate.autoscale=0
case (*pstate).pan of
	-1: begin
		move=(*pstate).pfrac*0.01*$
			((*pstate).winstate.xrange[1]-(*pstate).winstate.xrange[0])
		(*pstate).winstate.xrange=(*pstate).winstate.xrange-move
	end
	 1: begin
		move=(*pstate).pfrac*0.01*$
			((*pstate).winstate.xrange[1]-(*pstate).winstate.xrange[0])
	 	(*pstate).winstate.xrange=(*pstate).winstate.xrange+move
	end
	-2: begin
		move=(*pstate).pfrac*0.01*$
			((*pstate).winstate.yrange[1]-(*pstate).winstate.yrange[0])
		(*pstate).winstate.yrange=(*pstate).winstate.yrange-move
	end
 2: begin
		move=(*pstate).pfrac*0.01*$
			((*pstate).winstate.yrange[1]-(*pstate).winstate.yrange[0])
	 	(*pstate).winstate.yrange=(*pstate).winstate.yrange+move
	end
	else:
endcase
(*pstate).pan=0
;
;	If required, perform zooming operation.
if ((*pstate).zoom ne 0) then begin
	(*pstate).winstate.autoscale=0
	hwid=0.5*((*pstate).winstate.xrange[1]-(*pstate).winstate.xrange[0])
	hhgt=0.5*((*pstate).winstate.yrange[1]-(*pstate).winstate.yrange[0])
	xcen=0.5*((*pstate).winstate.xrange[1]+(*pstate).winstate.xrange[0])
	ycen=0.5*((*pstate).winstate.yrange[1]+(*pstate).winstate.yrange[0])
	zfrac=(*pstate).zfrac*0.01
	case (*pstate).zoom of
		-1: begin
		(*pstate).winstate.xrange=[xcen-hwid*zfrac,xcen+hwid*zfrac]
		(*pstate).winstate.yrange=[ycen-hhgt*zfrac,ycen+hhgt*zfrac]
		end
		 1: begin
		(*pstate).winstate.xrange=[xcen-hwid/zfrac,xcen+hwid/zfrac]
		(*pstate).winstate.yrange=[ycen-hhgt/zfrac,ycen+hhgt/zfrac]
		end
		else:
	endcase
endif
(*pstate).zoom=0
;
abscisrange=(*pstate).winstate.xrange
ordinarange=(*pstate).winstate.yrange
;
;;;;;;;wset,(*pstate).winstate.winPix
;
; Start plotting.
; First plot the frame, ticks, labels etc, but no data.
plot,abscis,ordina,Xstyle=1,Ystyle=1,Xtitle=samelabel,Ytitle="",$
	Xrange=abscisrange,Yrange=ordinarange,title=title,color=1,/nodata
;
nd=size(ordina,/n_dimensions)
;
for dataset=0,(*pstate).ndatasets-1 do begin
	if (not (*(*pstate).activePtr)[dataset]) then continue
	if ( ((*pstate).ndatasets eq 1 and nd eq 1) $
		or ((*pstate).ndatasets gt 1 and nd eq 2)) then begin
		ordina1=ordina[*,dataset];/(dataset+1) [only used for tests]
		oerror1=oerror[*,dataset];/(dataset+1) [only used for tests]
		oplot,abscis,ordina1,psym=-dave_selsym(0)*(*pstate).join_pts,$
			symsize=0.5,color=2+dataset
		if ((*pstate).plot_eb) then dave_cerrplot,abscis,ordina1-oerror1,$
			ordina1+oerror1,color=2+dataset
	endif else begin
		ordina1=ordina[*,*,dataset];/(dataset+1) [only used for tests]
		oerror1=oerror[*,*,dataset];/(dataset+1) [only used for tests]
		for k=0,(*pstate).ngrps-1 do begin
			oplot,abscis,ordina1[*,k],psym=-dave_selsym(k)*(*pstate).join_pts,$
				symsize=0.5,color=2+dataset
			if ((*pstate).plot_eb) then dave_cerrplot,abscis,ordina1[*,k]-oerror1[*,k],$
				ordina1[*,k]+oerror1[*,k],color=2+dataset
		endfor
	endelse
endfor
;
;;;;;; Copy to screen window.
;;;;;wset,(*pstate).winstate.winVis
;;;;;device,copy = [0,0,!d.x_size,!d.y_size,0,0,(*pstate).winstate.winPix]
;;;;;wshow,(*pstate).winstate.winVis
;
; Fill in abscissa and ordinate extrema.
widget_control,(*pstate).xlval,set_value=strcompress(!x.crange[0])
widget_control,(*pstate).xrval,set_value=strcompress(!x.crange[1])
widget_control,(*pstate).ydval,set_value=strcompress(!y.crange[0])
widget_control,(*pstate).yuval,set_value=strcompress(!y.crange[1])
;
widget_control,event.top,set_uvalue=pstate,/no_copy
;
end


;************************************************************************************************
pro dave_mdicerPrint,event
;************************************************************************************************
;
compile_opt strictarr
;
thisDevice = !d.name
deviceKeywords = ps_form(Cancel = canceled,parent = event.top)
if canceled ne 1 then begin
  SET_PLOT, 'PS'
  DEVICE, _Extra = deviceKeywords
  dave_mdicerPlot,event
  DEVICE, /CLOSE_FILE
  set_plot,thisDevice
endif
return
end


;************************************************************************************************
pro dave_mdicerSavedata,event
;************************************************************************************************
;
compile_opt strictarr
;
widget_control,event.top,get_uvalue = pState,/no_copy
(*pState).savedata=1
;
; Update the plot.
wset,(*pstate).winstate.winPix
widget_control,event.top,set_uvalue = pState,/no_copy
dave_mdicerPlot,event
widget_control,event.top,get_uvalue = pState,/no_copy
wset,(*pstate).winstate.winVis
device,copy = [0,0,!d.x_size,!d.y_size,0,0,(*pstate).winstate.winPix]
wshow,(*pstate).winstate.winVis
;
(*pState).savedata=0
widget_control,event.top,set_uvalue = pState,/no_copy
return
end


;************************************************************************************************
pro dave_mdicerDraw,event
;************************************************************************************************
;
compile_opt strictarr
;
widget_control,event.top,get_uvalue = pState,/no_copy
;
case event.type of
0:	begin		; button press
	  (*pState).winState.mouse = event.press
	  if (*pState).winState.mouse eq 4 then begin
	    (*pState).winState.autoscale = 1
			widget_control,event.top,set_uvalue = pState,/no_copy
	    return
	  endif
	  if (*pState).winState.mouse eq 1 then begin
	    (*pState).winState.xbox[0] = event.x
	    (*pState).winState.ybox[0] = event.y
	    wset,(*pState).winState.winVis
	    device,copy = [0,0,!d.x_size,!d.y_size,0,0,(*pState).winState.winPix]
	    empty
	    (*pState).winState.autoscale = 0
	  endif
	end
1:	begin	; button release
	  xll = (*pState).winState.xbox[0] < (*pState).winState.xbox[1]
	  yll = (*pState).winState.ybox[0] < (*pState).winState.ybox[1]
	  w = abs((*pState).winState.xbox[1] - (*pState).winState.xbox[0])
	  h = abs((*pState).winState.ybox[1] - (*pState).winState.ybox[0])
	  xur = xll + w
	  yur = yll + h
	  ll = convert_coord(xll,yll,/device,/to_data,/double)
	  ur = convert_coord(xur,yur,/device,/to_data,/double)
	  (*pState).winState.xrange = [ll[0],ur[0]]
	  (*pState).winState.yrange = [ll[1],ur[1]]
	  wset,(*pState).winState.winPix
	  widget_control,event.top,set_uvalue = pState,/no_copy
	  dave_mdicerPlot,event
	  widget_control,event.top,get_uvalue = pState,/no_copy
	  wset,(*pState).winState.winVis
	  device,copy = [0,0,!d.x_size,!d.y_size,0,0,(*pState).winState.winPix]
	  (*pState).winState.mouse = 0B
	end
2:	begin	; mouse motion
  	result=convert_coord(event.x,event.y,/device,/to_data)
		if (result[0] gt !x.crange[0] and result[0] lt !x.crange[1] and $
				result[1] gt !y.crange[0] and result[1] lt !y.crange[1] and $
				(*pstate).ndatasets gt 0) then $
		  widget_control,(*pState).cpos,set_value=["COORDINATES",string(result[0:1])] else $
		  widget_control,(*pState).cpos,set_value=""
	  if (*pState).winState.mouse eq 1 then begin
	  	(*pState).winState.xbox[1] = event.x
	  	(*pState).winState.ybox[1] = event.y
	  	xc = [(*pState).winState.xbox[0],event.x,event.x,(*pState).winState.xbox[0],$
	  	      (*pState).winState.xbox[0]]
	  	yc = [(*pState).winState.ybox[0],(*pState).winState.ybox[0],event.y,event.y,$
	  	      (*pState).winState.ybox[0]]
 	  	wset,(*pState).winState.winVis
	  	device,copy = [0,0,!d.x_size,!d.y_size,0,0,(*pState).winState.winPix]
	  	plots,xc,yc,/device,color=1
	  	empty
	  endif
	end
else:
endcase
;
widget_control,event.top,set_uvalue = pState,/no_copy
;
return
end


;************************************************************************************************
pro dave_mdicer_setops_handler,event
;************************************************************************************************
;
compile_opt strictarr
;
widget_control,event.top,get_uvalue=setupstate,/no_copy
;
if (event.id eq setupstate.done) then begin
	*(setupstate.ptr)=widget_info(setupstate.list,/list_select)
	widget_control,event.top,/destroy
	return
endif
;
widget_control,event.top,set_uvalue=setupstate,/no_copy
;
end


;************************************************************************************************
pro dave_mdicer_setops,event
;************************************************************************************************
;
compile_opt strictarr
;
widget_control,event.top,get_uvalue=pstate,/no_copy
;
; Obtain indices of active datasets. There must be at least one active dataset.
repeat begin
	tlb=widget_base(title="Select active sets (minimum 1)",$
		group_leader=event.top,/modal,/col)
	n=n_elements(*(*pstate).filesPtr)
	information=strcompress(string(indgen(n)+1))+"  "+*(*pstate).filesPtr
	list=widget_list(tlb,value=information,/multiple,ysize=8)
	done=widget_button(tlb,value='done')
	ptr=ptr_new(/allocate_heap)
	widget_control,list,set_list_select=where(*(*pstate).activePtr)
	widget_control,tlb,/realize
	setupstate={list:list,done:done,ptr:ptr}
	widget_control,tlb,set_uvalue=setupstate
	xmanager,'dave_mdicer_setops',tlb,event_handler='dave_mdicer_setops_handler'
	chosen=*(setupstate.ptr)
	ptr_free,ptr
	if (chosen[0] eq -1) then $
		res=dialog_message("At least one dataset must be active.",/error)
endrep until (chosen[0] ne -1)
;
case event.id of
	(*pstate).abutton: begin
		*(*pstate).activePtr=intarr((*pstate).ndatasets)
		(*(*pstate).activePtr)[chosen]=1
	end
;
	(*pstate).tbutton: begin
		res=dialog_message("Sorry. This menu item is not yet coded.")
	end
	else:
endcase
;
; Update the plot.
wset,(*pstate).winstate.winPix
widget_control,event.top,set_uvalue=pstate,/no_copy
dave_mdicerPlot,event
widget_control,event.top,get_uvalue=pstate,/no_copy
wset,(*pstate).winstate.winVis
device,copy = [0,0,!d.x_size,!d.y_size,0,0,(*pstate).winstate.winPix]
wshow,(*pstate).winstate.winVis
widget_control,event.top,set_uvalue=pstate,/no_copy
;
end


;************************************************************************************************
pro dave_mdicer_handler,event
;************************************************************************************************
;
compile_opt strictarr
;
widget_control,event.top,get_uvalue=pstate,/no_copy
;
case event.id of
;
; Handle error bars menu item.
	(*pstate).ebutton: begin
		(*pstate).plot_eb=1-(*pstate).plot_eb
		if ((*pstate).plot_eb) then $
			widget_control,(*pstate).ebutton,set_value="Hide error bars" $
		else widget_control,(*pstate).ebutton,set_value="Show error bars"
	end
;
; Handle join points menu item.
	(*pstate).pbutton: begin
		(*pstate).join_pts=-(*pstate).join_pts
		if ((*pstate).join_pts eq 1) then $
			widget_control,(*pstate).pbutton,set_value="Do not join points"
		if ((*pstate).join_pts eq -1) then $
			widget_control,(*pstate).pbutton,set_value="Join points"
	end
;
;	Handle plot vs 2theta and abs(2theta) menu items.
	(*pstate).tthbutt: begin
		ybutton=(*pstate).ybutton
		widget_control,event.top,set_uvalue=pstate,/no_copy
		dave_mdicer_handler,{top:event.top,id:ybutton,handler:0l}
		widget_control,event.top,get_uvalue=pstate,/no_copy
		(*pstate).plotv2th=1
		(*pstate).plota2th=0
		(*pstate).plotvst=0
		(*pstate).plotvsE=0
		(*pstate).plott4E=0
		(*pstate).winstate.autoscale=1
	end
	(*pstate).athbutt: begin
		ybutton=(*pstate).ybutton
		widget_control,event.top,set_uvalue=pstate,/no_copy
		dave_mdicer_handler,{top:event.top,id:ybutton,handler:0l}
		widget_control,event.top,get_uvalue=pstate,/no_copy
		(*pstate).plotv2th=0
		(*pstate).plota2th=1
		(*pstate).plotvst=0
		(*pstate).plotvsE=0
		(*pstate).plott4E=0
		(*pstate).winstate.autoscale=1
	end
;
;	Handle Plot dsdt vs T_SD, dsdt vs E transfer, t^4*dsdt vs E transfer menu items.
	(*pstate).itmbutt: begin
		xbutton=(*pstate).xbutton
		widget_control,event.top,set_uvalue=pstate,/no_copy
		dave_mdicer_handler,{top:event.top,id:xbutton,handler:0l}
		widget_control,event.top,get_uvalue=pstate,/no_copy
		(*pstate).plotv2th=0
		(*pstate).plota2th=0
		(*pstate).plotvst=1
		(*pstate).plotvsE=0
		(*pstate).plott4E=0
		(*pstate).winstate.autoscale=1
	end
	(*pstate).ienbutt: begin
		xbutton=(*pstate).xbutton
		widget_control,event.top,set_uvalue=pstate,/no_copy
		dave_mdicer_handler,{top:event.top,id:xbutton,handler:0l}
		widget_control,event.top,get_uvalue=pstate,/no_copy
		(*pstate).plotv2th=0
		(*pstate).plota2th=0
		(*pstate).plotvst=0
		(*pstate).plotvsE=1
		(*pstate).plott4E=0
		(*pstate).winstate.autoscale=1
	end
	(*pstate).it4butt: begin
		xbutton=(*pstate).xbutton
		widget_control,event.top,set_uvalue=pstate,/no_copy
		dave_mdicer_handler,{top:event.top,id:xbutton,handler:0l}
		widget_control,event.top,get_uvalue=pstate,/no_copy
		(*pstate).plotv2th=0
		(*pstate).plota2th=0
		(*pstate).plotvst=0
		(*pstate).plotvsE=0
		(*pstate).plott4E=1
		(*pstate).winstate.autoscale=1
	end
;
;	Handle "revert" menu item.
	(*pstate).revbutt: begin
		(*pstate).plotv2th=0
		(*pstate).plota2th=0
		(*pstate).plotvst=0
		(*pstate).plotvsE=0
		(*pstate).plott4E=0
		(*pstate).winstate.autoscale=1
	end
;
; Handle group center slider button.
	(*pstate).xyvalbut: begin
		if ((*pstate).plot_vs_x) then begin
			(*pstate).yvalue=event.value
		endif else begin
			(*pstate).xvalue=event.value
		endelse
	end
;
; Handle group half-width slider button.
	(*pstate).xyrngbut: begin
		width=event.value
		if ((*pstate).plot_vs_x) then (*pstate).ywidth=width
		if ((*pstate).plot_vs_y) then (*pstate).xwidth=width
	end
;
; Handle abscissa rebinning slider button.
	(*pstate).xyrbnbut: begin
		if ((*pstate).plot_vs_x) then begin
			(*pstate).xrebin=event.value
		endif else begin
			(*pstate).yrebin=event.value
		endelse
	end
;
; Handle menu items that determine the abscissa.
	(*pstate).xbutton: begin
		widget_control,(*pstate).xbutton,get_value=tmp
		if (strmid(tmp,0,1) ne '[') then begin
			widget_control,(*pstate).xbutton,set_value='['+tmp+']'
		endif
		widget_control,(*pstate).ybutton,get_value=tmp
		if (strmid(tmp,0,1) eq '[') then begin
			tmp=strmid(tmp,1,strlen(tmp)-2)
			widget_control,(*pstate).ybutton,set_value=tmp
		endif
		if (not (*pstate).plot_vs_x) then (*pstate).winstate.autoscale=1
		(*pstate).plot_vs_x=1
		(*pstate).plot_vs_y=0
		widget_control,(*pstate).xyvalbut,set_value=(*pstate).yvalue
		widget_control,(*pstate).xyrngbut,set_value=(*pstate).ywidth
		widget_control,(*pstate).xyrbnbut,set_value=(*pstate).xrebin
		widget_control,(*pstate).xyvalbut,set_slider_max=(*pstate).ydim
		widget_control,(*pstate).xyrngbut,set_slider_max=(*pstate).ydim
		widget_control,(*pstate).xyrbnbut,set_slider_max=(*pstate).xdim/10 > 6
		widget_control,(*pstate).grps,set_slider_max=(*pstate).ydim
	end

	(*pstate).ybutton: begin
		widget_control,(*pstate).ybutton,get_value=tmp
		if (strmid(tmp,0,1) ne '[') then begin
			widget_control,(*pstate).ybutton,set_value='['+tmp+']'
		endif
		widget_control,(*pstate).xbutton,get_value=tmp
		if (strmid(tmp,0,1) eq '[') then begin
			tmp=strmid(tmp,1,strlen(tmp)-2)
			widget_control,(*pstate).xbutton,set_value=tmp
		endif
		if (not (*pstate).plot_vs_y) then (*pstate).winstate.autoscale=1
		(*pstate).plot_vs_x=0
		(*pstate).plot_vs_y=1
		widget_control,(*pstate).xyvalbut,set_value=(*pstate).xvalue
		widget_control,(*pstate).xyrngbut,set_value=(*pstate).xwidth
		widget_control,(*pstate).xyrbnbut,set_value=(*pstate).yrebin
		widget_control,(*pstate).xyvalbut,set_slider_max=(*pstate).xdim
		widget_control,(*pstate).xyrngbut,set_slider_max=(*pstate).xdim
		widget_control,(*pstate).xyrbnbut,set_slider_max=(*pstate).ydim/10 > 6
		widget_control,(*pstate).grps,set_slider_max=(*pstate).xdim
	end
;
; Handle single group menu item.
	(*pstate).sgrpbut: begin
		widget_control,(*pstate).sgrpbut,get_value=tmp
		if (strmid(tmp,0,1) ne '[') then begin
			widget_control,(*pstate).sgrpbut,set_value='['+tmp+']'
		endif
		widget_control,(*pstate).mgrpbut,get_value=tmp
		if (strmid(tmp,0,1) eq '[') then begin
			tmp=strmid(tmp,1,strlen(tmp)-2)
			widget_control,(*pstate).mgrpbut,set_value=tmp
		endif
		widget_control,(*pstate).agrpbut,get_value=tmp
		if (strmid(tmp,0,1) eq '[') then begin
			tmp=strmid(tmp,1,strlen(tmp)-2)
			widget_control,(*pstate).agrpbut,set_value=tmp
		endif
		(*pstate).one=1
		(*pstate).some=0
		(*pstate).all=0
	end
;
; Handle multiple group menu item.
	(*pstate).mgrpbut: begin
		widget_control,(*pstate).mgrpbut,get_value=tmp
		if (strmid(tmp,0,1) ne '[') then begin
			widget_control,(*pstate).mgrpbut,set_value='['+tmp+']'
		endif
		widget_control,(*pstate).agrpbut,get_value=tmp
		if (strmid(tmp,0,1) eq '[') then begin
			tmp=strmid(tmp,1,strlen(tmp)-2)
			widget_control,(*pstate).agrpbut,set_value=tmp
		endif
		widget_control,(*pstate).sgrpbut,get_value=tmp
		if (strmid(tmp,0,1) eq '[') then begin
			tmp=strmid(tmp,1,strlen(tmp)-2)
			widget_control,(*pstate).sgrpbut,set_value=tmp
		endif
		(*pstate).one=0
		(*pstate).some=1
		(*pstate).all=0
	end
;
; Handle all-in-one group menu item.
	(*pstate).agrpbut: begin
		widget_control,(*pstate).agrpbut,get_value=tmp
		if (strmid(tmp,0,1) ne '[') then begin
			widget_control,(*pstate).agrpbut,set_value='['+tmp+']'
		endif
		widget_control,(*pstate).sgrpbut,get_value=tmp
		if (strmid(tmp,0,1) eq '[') then begin
			tmp=strmid(tmp,1,strlen(tmp)-2)
			widget_control,(*pstate).sgrpbut,set_value=tmp
		endif
		widget_control,(*pstate).mgrpbut,get_value=tmp
		if (strmid(tmp,0,1) eq '[') then begin
			tmp=strmid(tmp,1,strlen(tmp)-2)
			widget_control,(*pstate).mgrpbut,set_value=tmp
		endif
		(*pstate).one=0
		(*pstate).some=0
		(*pstate).all=1
	end
;
; Handle pan and zoom buttons ("direction" and magnitude)
	(*pstate).lpanbut: (*pstate).pan=-1
	(*pstate).rpanbut: (*pstate).pan=1
	(*pstate).dpanbut: (*pstate).pan=-2
	(*pstate).upanbut: (*pstate).pan=2
	(*pstate).zoinbut: (*pstate).zoom=-1
	(*pstate).zootbut: (*pstate).zoom=1
	(*pstate).pfrcbut: (*pstate).pfrac=event.value
	(*pstate).zfrcbut: (*pstate).zfrac=event.value
;
; Handle minimum and maximum abscissa value boxes.
	(*pstate).xlval: begin
		if (event.type eq 0) then begin
			widget_control,(*pstate).xlval,get_value=tmpval
			(*pstate).winstate.autoscale=0
			(*pstate).winstate.xrange[0]=tmpval
		endif
	end
	(*pstate).xrval: begin
		if (event.type eq 0) then begin
			widget_control,(*pstate).xrval,get_value=tmpval
			(*pstate).winstate.autoscale=0
			(*pstate).winstate.xrange[1]=tmpval
		endif
	end
;
; Handle minimum and maximum ordinate value boxes.
	(*pstate).ydval: begin
		if (event.type eq 0) then begin
			widget_control,(*pstate).ydval,get_value=tmpval
			(*pstate).winstate.autoscale=0
			(*pstate).winstate.yrange[0]=tmpval
		endif
	end
	(*pstate).yuval: begin
		if (event.type eq 0) then begin
			widget_control,(*pstate).yuval,get_value=tmpval
			(*pstate).winstate.autoscale=0
			(*pstate).winstate.yrange[1]=tmpval
		endif
	end
;
; Handle number of groups slider.
	(*pstate).grps: begin
		widget_control,event.id,get_value=ngrps
		(*pstate).ngrps=ngrps[0]
	end

	else:
endcase
;
; As needed, modify sensitivities of buttons.
if ((*pstate).one) then begin
	widget_control,(*pstate).xyvalbut,sensitive=1
	widget_control,(*pstate).xyrngbut,sensitive=1
	widget_control,(*pstate).gbase,sensitive=0
endif
if ((*pstate).some) then begin
	widget_control,(*pstate).xyvalbut,sensitive=0
	widget_control,(*pstate).xyrngbut,sensitive=0
	widget_control,(*pstate).gbase,sensitive=1
endif
if ((*pstate).all) then begin
	widget_control,(*pstate).xyvalbut,sensitive=0
	widget_control,(*pstate).xyrngbut,sensitive=0
	widget_control,(*pstate).gbase,sensitive=0
endif
;
; If there is something to plot, plot it.
doplot=(*pstate).ndatasets gt 0
if (doplot) then begin
	wset,(*pstate).winstate.winPix
	widget_control,event.top,set_uvalue=pstate,/no_copy
	dave_mdicerPlot,event
	widget_control,event.top,get_uvalue=pstate,/no_copy
	wset,(*pstate).winstate.winVis
	device,copy = [0,0,!d.x_size,!d.y_size,0,0,(*pstate).winstate.winPix]
	wshow,(*pstate).winstate.winVis
endif
;
widget_control,event.top,set_uvalue=pstate,/no_copy
;
end


;************************************************************************************************
pro dave_mdicer,notifyIDs,inPtr,group_leader = group_leader
;************************************************************************************************
; This is the widget definition module

compile_opt strictarr
;
;ptr_free,ptr_valid()
;show_ptrs,'start';
;
;;daveDeSensitizeButtons,event
;
;widget_control,event.top,get_uvalue = davePtr,/no_copy
;
dave_makeDavePtr,davePtr=davePtr
;
;show_ptrs,'makeDavePtr'
;
davesysvars
;
dcs_sysvars,davePtr=davePtr
;
;amcolor = {color, $ ; name
;red:   [255,0,255,  0,  0,  0,255,255,152,128,128,128,255,178,255],$ ; red values
;green: [255,0,  0,255,  0,255,  0,255,120,128,128,128, 54,192,128],$ ; green values
;blue:  [255,0,  0,  0,255,255,255,  0,103,220, 60,128, 54,220,128] $ ; blue values
;}
;colorset
;oldct=dcs_getctinfo()
;
; Save current color table.
;oldct=dcs_getctinfo()
; Use Alan's routine to load 256 colors.
tvlct,!dave_colors.red,!dave_colors.green,!dave_colors.blue
;
; Define top level widget.
tlb=widget_base(title="DAVE multiple dataset slicer",/col,mbar = bar)
;
; Define file menu.
filemenu = widget_button(bar,value = 'File',/menu)
	void = widget_button(filemenu,value = 'Select a DAVE data file',$
		event_pro = 'dave_mdicerRead',uname='readDAVE')
	void = widget_button(filemenu,value = 'Select a DCS raw data file',$
		event_pro = 'dave_mdicerRead',uname='readDCS')
	wprint = widget_button(filemenu,value = 'Print to a postscript file',$
		event_pro = 'dave_mdicerPrint',sensitive=0)
	wsplot = widget_button(filemenu,value = 'Save plot',$
		event_pro = 'dave_mdicerSaveplot',sensitive=0)
	wrplot = widget_button(filemenu,value = 'Restore plot',$
		event_pro = 'dave_mdicerRestoreplot')
	wsdata = widget_button(filemenu,value = 'Save plotted data',$
		event_pro = 'dave_mdicerSavedata',sensitive=0)
	void = widget_button(filemenu,value = 'Clear/reset',event_pro = 'dave_mdicerClear')
	void = widget_button(filemenu,value = 'Quit',event_pro = 'dave_mdicerQuit')
;
; Define abscissa menu.
xymenu=widget_button(bar,value='Abscissa',/menu,sensitive=0)
	xbutton=widget_button(xymenu,value='x: (not yet defined)')
	ybutton=widget_button(xymenu,value='y: (not yet defined)')
;
; Define grouping menu.
gpmenu=widget_button(bar,value='Grouping',/menu,sensitive=0)
	sgrpbut=widget_button(gpmenu,value='Single group')
	mgrpbut=widget_button(gpmenu,value='Several groups')
	agrpbut=widget_button(gpmenu,value='[All in one group]')
;
; Define options menu.
optmenu = widget_button(bar,value = 'Options',/menu,sensitive=0)
	ebutton = widget_button(optmenu,value = 'Show error bars')
	pbutton = widget_button(optmenu,value = 'Join points')
	rbutton = widget_button(optmenu,value = 'DCS raw data options',/menu,sensitive=0)
		tthbutt = widget_button(rbutton,value = 'Plot vs 2theta')
		athbutt = widget_button(rbutton,value = 'Plot vs abs(2theta)')
		itmbutt = widget_button(rbutton,value = 'Plot dsdt vs T_SD')
		ienbutt = widget_button(rbutton,value = 'Plot dsdt vs E transfer')
		it4butt = widget_button(rbutton,value = 'Plot t^4*dsdt vs E transfer')
		revbutt = widget_button(rbutton,value = 'Revert to normal plot')
;
; Define set operations menu.
setmenu = widget_button(bar,value = 'Set ops',/menu,sensitive=0)
	abutton = widget_button(setmenu,value = 'Activate',event_pro = 'dave_mdicer_setops')
	tbutton = widget_button(setmenu,value = 'Transform',event_pro = 'dave_mdicer_setops')
;
; Create widget sub-base plus pan and zoom buttons.
bas0=widget_base(tlb,/row,/align_center)
	col1=widget_base(bas0,/col,/frame)
		row1a=widget_base(col1,/row)
			lpanbut=widget_button(row1a,value="Pan left",sensitive=0)
			rpanbut=widget_button(row1a,value="Pan right",sensitive=0)
		row1b=widget_base(col1,/row)
			dpanbut=widget_button(row1b,value="Pan down",sensitive=0)
			upanbut=widget_button(row1b,value="Pan up",sensitive=0)
		geometry=widget_info(rpanbut,/geometry)
		xs=geometry.xsize
		widget_control,lpanbut,xsize=xs
		widget_control,rpanbut,xsize=xs
		widget_control,dpanbut,xsize=xs
		widget_control,upanbut,xsize=xs
		pfrcbut=widget_slider(col1,title="Pan percent",$
			minimum=0,maximum=90,value=20)
	col2=widget_base(bas0,/col,/frame)
		row2a=widget_base(col2,/row)
			zoinbut=widget_button(row2a,value="Zoom in",sensitive=0)
		row2b=widget_base(col2,/row)
			zootbut=widget_button(row2b,value="Zoom out",sensitive=0)
		geometry=widget_info(zootbut,/geometry)
		xs=geometry.xsize
		widget_control,zoinbut,xsize=xs
		widget_control,zootbut,xsize=xs
		zfrcbut=widget_slider(col2,title="Zoom percent",$
			minimum=10,maximum=100,value=80)
;
;	Create some sliders, plus text widgets that define x and y limits.
	bas1=widget_base(bas0,/col)
		xyrow=widget_base(bas1,/row,sensitive=1,/frame)
			xyvalbut=widget_slider(xyrow,title="Group center",minimum=1,value=1)
			xyrngbut=widget_slider(xyrow,title="Halfwidth of group",minimum=0,value=0)
			xyrbnbut=widget_slider(xyrow,title="Abscissa rebinning",minimum=1,value=1)
		corns=widget_base(bas1,/row,/frame)
			xlpos=widget_base(corns,/col)
				xllab=widget_label(xlpos,value='xmin')
				xlval=widget_text(xlpos,xsize=10,/editable)
			xrpos=widget_base(corns,/col)
				xrlab=widget_label(xrpos,value='xmax')
				xrval=widget_text(xrpos,xsize=10,/editable)
			ydpos=widget_base(corns,/col)
				ydlab=widget_label(ydpos,value='ymin')
				ydval=widget_text(ydpos,xsize=10,/editable)
			yupos=widget_base(corns,/col)
				yulab=widget_label(yupos,value='ymax')
				yuval=widget_text(yupos,xsize=10,/editable)
;
;	Create some more widgets.
	bas2=widget_base(bas0,/col)
	gbase=widget_base(bas2,sensitive=0,/frame,/align_center)
	grps=widget_slider(gbase,title="Number of groups",minimum=1,value=1)
	void=widget_base(bas2,ysize=4) ; (spacer)
	cpos=widget_text(bas2,ysize=3);,/editable)
;
; Create an information section.
	infobase=widget_base(tlb,/row,/align_center)
	info=widget_text(infobase,xsize=104,ysize=2,/editable,/scroll)
;
; Create the plot window itself.
geometry=widget_info(tlb,/geometry)
winxsize=geometry.xsize
winysize=300
;
	bottom=widget_base(tlb,/row)
		win=widget_draw(bottom,xsize=winxsize,ysize=winysize,$
			/button_events,/motion_events,event_pro="dave_mdicerdraw")
;
; Center the widget and realize it.
centertlb,tlb
widget_control,tlb,/realize
widget_control,xyvalbut,sensitive=0
widget_control,xyrngbut,sensitive=0
;
; Create a pixmap window for later use, the same size as the screen window.
window,/free,/pixmap,xsize=winxsize,ysize=winysize
winPix = !d.window
widget_control,win,get_value = winVis
;
; Define winstate.
winState =  {winPix:winPix,$
             winVis:winVis,$
             win:win,$
             xbox:[0.0,1.0],$
             xrange:[0.0d0,1.0d0],$
             ybox:[0.0,1.0],$
             yrange:[0.0d0,1.0d0],$
             xPtr:ptr_new(/allocate_heap),$
             yPtr:ptr_new(/allocate_heap),$
             xtitle:'',$
             ytitle:'',$
             title:'',$
             mouse:0B,$
             autoscale:1}
;
; Define the state variable.
state={winstate:winstate,$
; Widget IDs
	wprint:wprint,$
	wsplot:wsplot,$
	wrplot:wrplot,$
	wsdata:wsdata,$
	xymenu:xymenu,$
	gpmenu:gpmenu,$
	optmenu:optmenu,$
	setmenu:setmenu,$
	xbutton:xbutton,$
	ybutton:ybutton,$
	sgrpbut:sgrpbut,$
	mgrpbut:mgrpbut,$
	agrpbut:agrpbut,$
	ebutton:ebutton,$
	pbutton:pbutton,$
	rbutton:rbutton,$
	tthbutt:tthbutt,$
	athbutt:athbutt,$
	itmbutt:itmbutt,$
	ienbutt:ienbutt,$
	it4butt:it4butt,$
	revbutt:revbutt,$
	abutton:abutton,$
	tbutton:tbutton,$
	xyvalbut:xyvalbut,$
	xyrngbut:xyrngbut,$
	xyrbnbut:xyrbnbut,$
	lpanbut:lpanbut,$
	rpanbut:rpanbut,$
	dpanbut:dpanbut,$
	upanbut:upanbut,$
	zoinbut:zoinbut,$
	zootbut:zootbut,$
	zfrcbut:zfrcbut,$
	pfrcbut:pfrcbut,$
	xyrow:xyrow,$
	gbase:gbase,$
	grps:grps,$
	ngrps:1,$
	info:info,$
	cpos:cpos,$
	xlval:xlval,$
	xrval:xrval,$
	ydval:ydval,$
	yuval:yuval,$
; Initial values
	readingDCS:1,$
	xvalue:1,$
	yvalue:1,$
	xwidth:0,$
	ywidth:0,$
	xrebin:1,$
	yrebin:1,$
	plot_vs_x:1,$
	plot_vs_y:0,$
	plot_eb:0,$
	join_pts:-1,$
	plotv2th:0,$
	plota2th:0,$
	plotvst:0,$
	plotvsE:0,$
	plott4E:0,$
	one:0,$
	some:0,$
	all:1,$
	pan:0,$
	pfrac:20,$
	zoom:0,$
	zfrac:80,$
	xdim:1,$
	ydim:1,$
;	Defined when data are read in.
	pqty:ptr_new(/allocate_heap),$
	perr:ptr_new(/allocate_heap),$
	px:ptr_new(/allocate_heap),$
	py:ptr_new(/allocate_heap),$
	xlabel:'',$
	ylabel:'',$
	histlabel:'',$
	xunits:'',$
	yunits:'',$
	histunits:'',$
	xtype:'',$
	ytype:'',$
	tchansPtr:ptr_new(/allocate_heap),$
	timesPtr:ptr_new(/allocate_heap),$
	energiesPtr:ptr_new(/allocate_heap),$
	elastic_time:4000.0,$
	ndatasets:0,$
	activePtr:ptr_new(/allocate_heap),$
	filesPtr:ptr_new(/allocate_heap),$
	notifyIDs:notifyIDs,$
	savedata:0,$
	inPtr:inPtr}
;
widget_control,tlb,set_uvalue=ptr_new(state),/no_copy
;
;show_ptrs,'xmanager'
;
; xmanage.
xmanager,'dave_mdicer',tlb,event_handler='dave_mdicer_handler',$
	cleanup='dave_mdicerCleanup'
;
;;daveSensitizeButtons,event
end