;$Id: trifenestra_file.pro,v 1.7 2002/05/23 03:20:13 jmspark Exp $
;*******************************************************************************
;23456789 123456789 123456789 123465798 123456789 123456789 123465789 123456789
;       10        20        30        40        50        60        70        80
;*******************************************************************************
pro trifenestra::_open, files

	if obj_valid( self.oRaw ) then obj_destroy, self.oRaw
	self -> widget_control, 'rawButton', sensitive = 0

	; Select the appropriate default filter.
	if !version.os_family eq 'Windows' then $
		filter = '*.*' $
	else filter = '*'

	if !version.release ge 5.4 then $
		filter = $
			[ '*.dave', '*.dave.gz'  $
			, '*.spe', '*.spe.gz'  $
			, '*.ng5', '*.ng5.gz' ]

	if !trifenestra.internal then $
		filter = [ filter, '*.dcs', '*.dcs.gz' ]

	; Choose files.
	if n_elements( files ) eq 0 then $
		files = dave_dialog_pickfile( /read $
			, /multiple_files $
			, title = 'Please select the data files to read.' $
			, filter = filter $
			)
	if files[0] eq '' then return

	n_files = n_elements( files )
	if n_files eq 1 then files = files[0]

	; for i = 0, n_files - 1 do begin

		; Decide the file type.
		strings = strsplit( files[0], '.', /extract )
		n_strings = n_elements( strings )
		key = strings[ n_Strings - 1 ]

		; If it is a zipped file, select the next string as the key.
		if strlowcase( key ) eq 'gz' and n_strings ge 2 then $
			key = strings[ n_elements( strings ) - 2 ]

		case strlowcase( key ) of
			'dave': begin
				self -> file_read_dave, file = files
			end
			'spe': begin
				self -> file_read_spe, file = files
			end
			'ng5': begin
				self -> file_read_icp,  file = files
			end
			'dcs': begin
				self -> file_read_dcs, file = files
			end
			else: begin
				void = dialog_message( /error $
					, 'The file is not of a known type.' $
					)
				return
			end
		endcase

	; endfor

end
;*******************************************************************************
;23456789 123456789 123456789 123465798 123456789 123456789 123465789 123456789
;       10        20        30        40        50        60        70        80
;*******************************************************************************
;*******************************************************************************
;23456789 123456789 123456789 123465798 123456789 123456789 123465789 123456789
;       10        20        30        40        50        60        70        80
;*******************************************************************************
;
; 'FILE' menu event handlers
;
;*******************************************************************************
;23456789 123456789 123456789 123465798 123456789 123456789 123465789 123456789
;       10        20        30        40        50        60        70        80
;*******************************************************************************
;
; All printing events go through this.
;
pro pmenu_event,event
	widget_control,event.top,get_uvalue=self
	self -> pfigures,event
	return
end
;*******************************************************************************
;
; Main event handler for the printing events.
;
pro trifenestra::pfigures,event

	; widget_control,/hourglass
	widget_control, event.top, sensitive = 0

	; Save the current state.
	idl_state = idl_state_save()
	thisdevice = !d.name
	; thisfont = !p.font
	; thickness = !p.thick
	; device,get_visual_depth=thedepth
		; It is not clear why this line is here.

	; Branch off according to the kind of events.
	widget_control,event.id,get_uvalue=uval
	case uval of
		'pimage_pr': begin
			ok = dialog_printersetup()
			if not ok then break
			set_plot,'printer',/copy
			; !trifenestra.mainWindowState.device_name = !d.name
			device,/landscape;,/color;_extra=keywords
			self -> pimage;,event
			device,/close_document
		end
		'pivsx_pr': begin
			ok = dialog_printersetup()
			if not ok then break
			; keywords = pswindow(/printer,fudge=0.25,/landscape)
			set_plot,'printer',/copy
			device,/landscape;,/printer;_extra=keywords
			self -> pivsx;,event
			device,/close_document
		end
		'pivsy_pr': begin
			ok = dialog_printersetup()
			if not ok then break
			; keywords = pswindow(/printer,fudge=0.25,/landscape)
			set_plot,'printer',/copy
			device,/landscape;,/printer;,_extra=keywords
			self -> pivsy;,event
			device,/close_document
		end
		'pimage_ps': begin
			ofile = dialog_pickfile(/write,filter=['*.ps']	$
				,dialog_parent=event.top)
			if strcompress(strmid(ofile,strlen(ofile)-3,3)) eq '' then break
			set_plot,'ps',/copy
			device,/landscape,file=ofile,/color;_extra=keywords
			self -> pimage;,event
			device,/close
		end
		'pivsx_ps': begin
			ofile = dialog_pickfile(/write,filter=['*.ps']	$
				,dialog_parent=event.top)
			if strcompress(strmid(ofile,strlen(ofile)-3,3)) eq '' then break
			set_plot,'ps',/copy
			device,/landscape,file=ofile;,/printer;_extra=keywords
			loadct,0
			self -> pivsx;,event
			device,/close
			loadct,4
		end
		'pivsy_ps': begin
			ofile = dialog_pickfile(/write,filter=['*.ps']	$
				,dialog_parent=event.top)
			if strcompress(strmid(ofile,strlen(ofile)-3,3)) eq '' then break
			set_plot,'ps',/copy
			device,/landscape,file=ofile;,/printer;,_extra=keywords
			loadct,0
			self -> pivsy;,event
			device,/close
			loadct,4
		end
		else: print,'no match'

	end

	; Restore the original state.
	idl_state_restore, idl_state

	set_plot,thisdevice
	;!p.font = thisfont
	;!p.thick = thickness

	; Give the control back.
	widget_control, event.top, /sensitive

	return
end
;*******************************************************************************
;
; Printing the main window.
;
pro trifenestra::pimage;,event

	; Restore the setting for this window.
	idl_state_restore, !trifenestra.mainWindowState
	; !p = *self.bangp
	; !x = *self.bangx
	; !y = *self.bangy
	; !z = *self.bangz

	!p.charsize = 2
	!p.color = 0

	; Draw the image.
	self.oDave -> view, pos= !trifenestra.printerPosition $
		  , xrange=!x.range, yrange=!y.range, zrange=!z.range $
		  , /print

	; maketable
		; What is this?
		; Perhaps it is to generate the color table!


	nColors = 255
	x = [0,1]
	y = findgen( nColors ) / ( nColors - 1. ) $
		* (!z.range[1] - !z.range[0]) + !z.range[0]
	z = fltarr(2, nColors )
	z[0,*] = y
	z[1,*] = y

	!p.noerase = 1

	oCT = obj_new('oDave')
	void = oCT -> set( qty = z, xdata = x, ydata = y )
	oCT -> view, xrange=[0,1], yrange=!z.range $
       , xtit=' ',ytit=' ' $
       , pos=[.85,.15,.89,.85] $
       , xticks = 1 $
       , ystyle = 4 $
       , ytickname = replicate( ' ', 16 ) $
       , xtickname = replicate( ' ', 16 ) ; 16 is an arbitrary #.
	obj_destroy, oCT

	; Draw the Y axis on the right hand side of the color bar.
	axis, yaxis=1, yrange=!z.range, chars=2
	!p.noerase = 0

	return
end
;*******************************************************************************
;
; Printing the I vs X window.
;
pro trifenestra::pivsx;,event

	idl_state_restore, !trifenestra.bottomPanelState
	!p.color = 0

	plot, *self.x_bp, *self.y_bp $
		, color=0 $
		, psym = self.ix_plot_symbol $
		, chars=2 $

		, xtitle = self.xtitle $
		, xstyle = 1 $
		; , xrange = !x.range $ ; Must have been restored.
		; , xticks=3	$

		, ytitle='I' $
		, ystyle=1
		; , yrange = !y.range $ ; Must have been restored.

	errplot, *self.x_bp $
		, *self.y_bp - *self.dy_bp $
		, *self.y_bp + *self.dy_bp

	return
end
;*******************************************************************************
pro trifenestra::pivsy;,event

	idl_state_restore, !trifenestra.leftPanelState

		!x.range = !trifenestra.leftPanelState.bang_y.range
		!y.range = !trifenestra.leftPanelState.bang_x.range
		!p.color = 0

	plot,*self.x_lp,*self.y_lp,color=0  	$
		,yran=!y.range,xtitle=self.ytitle,ytitle='I' $;,xticks=3	$
		,xran=!x.range,psym=6,chars=2
	errplot,*self.x_lp,*self.y_lp-*self.dy_lp,*self.y_lp+*self.dy_lp

	return
end

;*******************************************************************************
;23456789 123456789 123456789 123465798 123456789 123456789 123465789 123456789
;       10        20        30        40        50        60        70        80
;*******************************************************************************
;
; All Saving Events Go Through This.
;
pro smenu_event,event
	widget_control,event.top,get_uvalue=self
	self -> wdata,event
	return
end
;*******************************************************************************
;
pro trifenestra::wdata, event

	widget_control,event.top,get_uvalue=self
	widget_control,event.id,get_uvalue=uval

	ofile = dave_dialog_pickfile( /write $
		, filter=['*.spe' ]	$
		, dialog_parent = event.top )

	if ofile eq '' then return

	case uval of

		'simage': begin
			; if n_elements(*self.intensity) lt 2 then begin

			if not self.oDave -> valid() then begin
			 	errarray = ['Error caught: No data to save']
				a = dialog_message(errarray,/error)
				return
			endif

			void = self.oDave -> write_spe( file = ofile )

	 		; sizeq = size(*self.q)
	 		; sizehw = size(*self.hw)

	 		; if sizeq[0] eq 1 then begin
	 			; trifenestra_write_spe,ofile,*self.q,*self.hw,*self.intensity,*self.error $
	 			;	,tit=self.title,xtit=self.xtitle,ytit=self.ytitle
	 		;	break
	 		; endif

			; q has bigger than 1 dimension.
	 		; if (n_elements(*self.angle) gt 1) and sizehw[0] eq 1 then $
	 	  	;	trifenestra_write_spe, ofile, *self.angle, *self.hw $
	 	  	;		, *self.intensity,*self.error	$
		 	;		, tit=self.title, xtit='Angle (degree)' $
		 	;		, ytit=self.ytitle $
			; else if (n_elements(*self.angle) gt 1) and sizehw[0] eq 2 then $
	 		;		trifenestra_write_spe,ofile,*self.angle,reform(*self.hw[0,*]) $
	 		;			,*self.intensity,*self.error $
	 		;			,tit=self.title,xtit='Angle (degree)' $
	 		;			,ytit=self.ytitle	$
	 		;	 else begin
		 	;			errarray = ['Error caught while saving the image data' $
		 	;				,'Please contact jmspark@nist.gov']
			;			a = dialog_message(errarray,/error)
			;			return
			 ;	endelse

	  	end ; 'simage'

		'sivsx': begin
			if n_elements(*self.x_bp) lt 2 then begin
				errarray = ['Error caught: No data to save']
				a = dialog_message(errarray,/error)
				return
			endif else begin
	 			if strmatch(self.xtitle,'*Q*',/fold_case) then	$
	  			xtitle = 'Q (A-1)' else 	$
	  			if strmatch(self.xtitle,'*!7x*',/fold_case) then $
	  			xtitle = 'hw (meV)' else xtitle=self.xtitle
	 			head = xtitle+'	Intensity	Error'

	 			trifenestra_wrc,*self.x_bp,*self.y_bp,*self.dy_bp,ofile,head=head $
	 				, format = '(3(E15.5))'
			endelse
		end ; 'sivsx'

		'sivsy': begin
			if n_elements(*self.x_lp) lt 2 then begin
				errarray = ['Error caught: No data to save']
				a = dialog_message(errarray,/error)
				return
			endif else begin
	 			if strmatch(self.ytitle,'*Q*',/fold_case) then $
	  			xtitle = 'Q (A-1)' else 	$
	  			if strmatch(self.ytitle,'*!7x*',/fold_case) then $
	  			xtitle = 'hw (meV)' else xtitle=self.ytitle
	 			head = xtitle+'	Intensity	Error'
	 			trifenestra_wrc,*self.x_lp,*self.y_lp,*self.dy_lp,ofile,head=head $
	 				, format = '(3(E15.5))'
			endelse
		end ; 'sivsy'
endcase
return
end




;*******************************************************************************
;23456789 123456789 123456789 123465798 123456789 123456789 123465789 123456789
;       10        20        30        40        50        60        70        80
;*******************************************************************************
;
; Preparation befor & Cleanup after reading data files.
;
;*******************************************************************************
pro trifenestra::file_ready_objects

	self.xrebin = 1
	self.yrebin = 1
	self -> widget_control, 'xrebin', set_value = self.xrebin
	self -> widget_control, 'yrebin', set_value = self.yrebin

	if obj_valid( self.oDave ) then begin
		self.oDave -> empty
	endif else begin
		self.oDave = obj_new('oDave')
	endelse

end
;*******************************************************************************
pro trifenestra::file_copy_objects

	if not obj_valid( self.oDaveCopy ) then self.oDaveCopy = obj_new('oDave')
	void = self.oDaveCopy -> set( self.oDave )

end
;*******************************************************************************
;23456789 123456789 123456789 123465798 123456789 123456789 123465789 123456789
;       10        20        30        40        50        60        70        80
;*******************************************************************************
;
; 'Reading' Group
;
;*******************************************************************************
pro trifenestra::file_open, event

	self -> _open

end
;*******************************************************************************
pro trifenestra::file_read_spe, file = file

	self -> file_ready_objects

	if not self.oDave -> read_spe( file = file ) then begin
		message, 'Reading SPE failed.', /continue
		return
	endif

	self -> file_copy_objects
	self -> convert_oDave2internal, self.oDave


end
;*******************************************************************************
pro trifenestra::file_read_dave, file = file

	self -> file_ready_objects

	if not self.oDave -> read_dave( file = file ) then begin
		message, 'Reading failed.', /continue
		return
	endif

	self -> file_copy_objects
	self -> convert_oDave2internal, self.oDave


end
;*******************************************************************************
pro trifenestra::file_read_icp, file = file
	; Currently we can only read SPINS Area Detector Data with this routine.
	; This must be changed in the future.

	self -> rspinsdata, file = file

end
;*******************************************************************************
pro trifenestra::file_read_dcs, file = file

	self -> file_ready_objects

	if not self.oDave -> read_dcs(file = file) then begin
		message, 'No valid DCS file was selected.', /continue
		return
	endif

	self -> file_copy_objects
	self -> convert_oDave2internal, self.oDave


end
;*******************************************************************************
;23456789 123456789 123456789 123465798 123456789 123456789 123465789 123456789
;       10        20        30        40        50        60        70        80
;*******************************************************************************
;
; 'Writing' Group
;
;*******************************************************************************
pro trifenestra::file_write_spe, event

	oTrife = self -> convert_Internal2oTrife()
	self -> write_spe, event, oTrife
	obj_destroy, oTrife

end
;*******************************************************************************
pro trifenestra::file_write_dave, event

	; self -> psd2workspace, event

	; Rob's favorite line:
	; widget_control,event.top,get_uvalue = pState,/no_copy

	; pState = self.daveStatePtr

	filename = DIALOG_PICKFILE(dialog_parent = event.top $
		, title = 'Enter filename' $
		, /write ,filter = '*.dave')

	if filename eq '' then begin
		; Rob's favorite line:
  		; widget_control,event.top,set_uvalue = pState,/no_copy
  		return
	endif

	void = self.oDave -> write_dave( file = filename )

	;filename = filename+'.dave'
	;state = *(*pState).inPtr
	;davePtr = ptr_new(state,/no_copy)
	;save, davePtr, FILENAME = filename

	; Is the following line necessary?
	; What happens to inPtr?
	; ptr_free,davePtr
	; Rob's favorite line:
	; widget_control,event.top,set_uvalue = pState,/no_copy

end
;*******************************************************************************
;23456789 123456789 123456789 123465798 123456789 123456789 123465789 123456789
;       10        20        30        40        50        60        70        80
;*******************************************************************************
;
; 'Printing' Group
;
;*******************************************************************************
pro trifenestra::file_print

end
;*******************************************************************************
;23456789 123456789 123456789 123465798 123456789 123456789 123465789 123456789
;       10        20        30        40        50        60        70        80
;*******************************************************************************
;
; 'Exit' Group
;
pro trifenestra::file_exit, event

	self -> dismiss, event

end
;*******************************************************************************
pro trifenestra::file, event
	; Do nothing.
end
;*******************************************************************************
;
; Wrapper for 'File' menu events.
;
function trifenestra_file, menubar

	wfilemenu = widget_button( menubar $
		, value='File' $
		, /menu $
		, event_pro = 'trifenestra_event')

	wOpen = widget_button( wfilemenu $
		, value = 'Open...' $
		, uvalue = 'file_open' $
		)


; Writing
wVoid = widget_button( wfilemenu $
		, value = 'Save as a .DAVE file...' $
		, uvalue = 'file_write_dave' $
		, /separator $
		)

wVoid = widget_button( wfilemenu $
		, value = 'Save as a .SPE file...' $
		, uvalue = 'file_write_spe' $
		)

simage= widget_button(wfilemenu $
	, value='Save as a classic SPE file...' $
	, uvalue='simage' $
	, event_pro = 'smenu_event')

sivsx = widget_button(wfilemenu $
	, value='Save I(X) as a TXT file...' $
	, uvalue='sivsx' $
	, event_pro = 'smenu_event')

sivsy = widget_button(wfilemenu $
	, value='Save I(Y) as a TXT file...' $
	, uvalue='sivsy' $
	, event_pro = 'smenu_event')


button = widget_button(wfilemenu $
	, /separator $
	, value='Print Contour Map' $
	, uvalue='pimage_pr' $
	, event_pro = 'pmenu_event')

button = widget_button( wfilemenu $
	, value='Print I(X)' $
	, uvalue='pivsx_pr' $
	, event_pro = 'pmenu_event' $
	)
button = widget_button(wfilemenu $
	, value='Print I(Y)' $
	, event_pro = 'pmenu_event' $
	, uvalue='pivsy_pr')

button = widget_button(wfilemenu $
	, value='Print to File: Contour Map' $
	, uvalue='pimage_ps' $
	, event_pro = 'pmenu_event' $
	)

button = widget_button(wfilemenu $
	, value='Print to File: I(X)' $
	, uvalue='pivsx_ps' $
	, event_pro = 'pmenu_event' $
	)
button = widget_button(wfilemenu $
	, value='Print to File: I(Y)'$
	, event_pro = 'pmenu_event' $
	, uvalue='pivsy_ps')


; Exit
wexit = widget_button(wfilemenu $
	, value='Exit' $
	, uvalue='file_exit' $
	, /separator)


return, wfilemenu

end
;*******************************************************************************