;$Id: trifenestra_transformbase.pro,v 1.1 2002/07/03 20:26:06 jmspark Exp $
;*******************************************************************************
;23456789 123456789 123456789 123465798 123456789 123456789 123465789 123456789
;       10        20        30        40        50        60        70        80
;*******************************************************************************
;
;	TRANSPOSE
;
;*******************************************************************************
pro trifenestra::transpEvent, event

	; Check the existence of the data.
	if not self.oDave -> valid() then return

	; Tranpose.
	self.oDave -> transpose
	self.oDaveCopy -> transpose

	; Put new labels.
	self.oDave -> get_label, xlabel = xlabel, ylabel = ylabel
	xtitleid = 'xtitle'
	ytitleid = 'ytitle'
	self -> widget_control,xtitleid,set_value=xlabel
	self -> widget_control,ytitleid,set_value=ylabel

	; Put new ranges.
	self.oDave -> get_range, xrange = xrange, yrange = yrange

	!trifenestra.mainWindowState.bang_x.range = xrange
	!trifenestra.mainWindowState.bang_y.range = yrange
	!trifenestra.bottomPanelState.bang_x.range = xrange
	!trifenestra.leftPanelState.bang_y.range = yrange

	; Transpose the rebin values.
	xrebinid = 'xrebin'
	yrebinid = 'yrebin'
	self -> widget_control, xrebinid, get_value = xrebin
	self -> widget_control, yrebinid, get_value = yrebin
	self.xrebin = yrebin
	self.yrebin = xrebin
	self -> widget_control, xrebinid, set_value = self.xrebin
	self -> widget_control, yrebinid, set_value = self.yrebin

	; Transpose the smoothing values.
	xsmoothid = 'xsmooth'
	ysmoothid = 'ysmooth'
	self -> widget_control, xsmoothid, get_value = xsmooth
	self -> widget_control, ysmoothid, get_value = ysmooth
	self.xsmooth = ysmooth
	self.ysmooth = xsmooth
	self -> widget_control, xsmoothid, set_value = self.xsmooth
	self -> widget_control, ysmoothid, set_value = self.ysmooth

	; Done. Draw.
	self -> updateImage

	return
end
;*******************************************************************************
;23456789 123456789 123456789 123465798 123456789 123456789 123465789 123456789
;       10        20        30        40        50        60        70        80
;*******************************************************************************
pro trifenestra::rawButton, event

	self -> file_ready_objects

	case self.isRaw of

		0: begin

			if not self.oDave -> set( self.oRaw ) then begin
				void = dialog_message(/error, 'Unable to perform the command.')
				return
			endif

		end
		1: begin

			if not self.oDave -> set( self.oDaveCopy ) then begin
				void = dialog_message(/error, 'Unable to perform the command.')
				return
			endif

		end
	endcase

	self -> convert_oDave2internal, self.oDave
	self.isRaw = 1b - self.isRaw
	self -> widget_control, 'rawButton' $
		, set_value = self.rawButtonString[ self.isRaw ]


end
;*******************************************************************************
;23456789 123456789 123456789 123465798 123456789 123456789 123465789 123456789
;       10        20        30        40        50        60        70        80
;*******************************************************************************
;
; Event Handler for the 'Q to Angle' button
;
; pro qtoangle,event
;
; widget_control,event.top,get_uvalue=self
; self -> qtoangleEvent
; return
;
; end
;*******************************************************************************
pro trifenestra::qtoangleEvent

	if n_elements(*self.angle) lt 2 then begin
		errarray = ['Error caught: No Angle available']
		a = dialog_message(errarray,/error)
		return
	endif

	; Find the widget ID of the q to angle button.
	qtoangleid ='qtoangle'
	self -> widget_control,qtoangleid,get_value=qorangle
	self.qorangle = qorangle[0]
		; self.qorangle now stores the lable of the button.

	; An example of a bad programming: the string is hard coded.
	if strmatch(self.qorangle,'Q -> Angle') then begin

		self.qorangle = 'Angle -> Q'
		ws	=	size(*self.hw)

		size_angle = size(*self.angle)

	case 1 of
		(ws[0] eq 2 and size_angle[0] eq 1): begin

			nw	=	n_elements(*self.hw[0,*])
			oned	=	1.+fltarr(nw)
			*self.q	=	*self.angle # oned
				; It looks to me that the angle data is stored in a 1D array.
			end
		else: *self.q = *self.angle
	endcase

	self.xtitle = 'Angle (degree)'
	self.title = 'I (Angle,hw)'

endif else begin

	self.qorangle = 'Q -> Angle'
	*self.q = *self.qCopy
	!trifenestra.mainWindowState.bang_x.range = $
		[min(*self.q),max(*self.q)]
	self.xtitle = 'Q (A-1)'
	self.title = 'I (Q,hw)'

end

; Change the plot ranges.
(*self.bangx).range = [min(*self.q),max(*self.q)]
(*self.bangx_bp).range = [min(*self.q),max(*self.q)]

; Display the corrected titles.
xtitleid = 'xtitle'
self -> widget_control,xtitleid,set_value=self.xtitle
titleid = 'title'
self -> widget_control,titleid,set_value=self.title

; Change the text in the button.
self -> widget_control,qtoangleid,set_value=self.qorangle

; Update the image.
self -> updateImage

return
end

; -------------------------------------------------------
; |	plot Log (Intensity)			|
; -------------------------------------------------------
;pro dologscale,event
;widget_control,event.top,get_uvalue=self
;self -> dologscaleEvent
;return
;end

pro trifenestra::dologscaleEvent, event

	strings = [ 'I -> Log(I)', 'Log(I) -> I' ]

	if not self.oDave -> valid() then return

	; Change the status bit.
	self.log_scale = 1 - self.log_scale

	; Change the label of the button.
	log_scaleid = 'log_scale'
	self -> widget_control, log_scaleid $
		, set_value = strings[ self.log_scale ]

	zminid = 'zmin'
	zmaxid = 'zmax'

	if self.log_scale then begin
		self -> widget_control,zminid,get_value=zmin
		self -> widget_control,zmaxid,get_value=zmax

	; If both zmin and zmax are negative, it causes an error.
	; If zmax is positive while zmin is not, zmin is changed into
	; a very small positive number.
	if zmin[0] le 0 then begin
		if zmax[0] gt 0 then zmin[0]=zmax[0]*1e-20	$
			else begin
			  errarray = ['Error caught: zmax < 0']
			  a = dialog_message(errarray,/error)
			return
			endelse
	endif

	self.oDave -> get, qty = qty, err = err

	lisneg = where(qty le 0,nneg) ; Indices of negative intensities.
	lispos = where(qty gt 0,npos) ; Indices of positive intensities.
	if nneg gt 0 then qty[lisneg] = alog10(zmin[0])
		; Those negative intensities are converted into a logarithm of
		; a very small number.
	if npos gt 0 then qty[lispos] = alog10(qty[lispos])
		; Regular positive numbers are 'log'ed.
	lisneg = where(err le 0,nneg)
	lispos = where(err gt 0,npos)
	if nneg gt 0 then err[lisneg] = 0
	if npos gt 0 then err[lispos] = alog10(err[lispos])
		; Error bars are converted into logarithms too.
		; Is it a correct procedure?

	void = self.oDave -> set( qty = qty, err = err )

	endif else begin

		void = self.oDave -> set( self.oDaveCopy )
		self.oDave -> get, qty = qty

	endelse

	qty_min = min( qty, max = qty_max )
	self -> widget_control,zminid,set_value= qty_min
	self -> widget_control,zmaxid,set_value= qty_max

	self -> updateImage

	return

end
;*******************************************************************************
;23456789 123456789 123456789 123465798 123456789 123456789 123465789 123456789
;       10        20        30        40        50        60        70        80
;*******************************************************************************

;*******************************************************************************
;23456789 123456789 123456789 123465798 123456789 123456789 123465789 123456789
;       10        20        30        40        50        60        70        80
;*******************************************************************************
function trifenestra_transformBase, base

	; Define the Base.
	transformBase =  widget_base( base $
		, /frame $
		, /col $
		, uname = 'transformbase' $
		, event_pro = 'trifenestra_event')


	wLabel = widget_label( transformBase $
		, value = 'Transformation' $
		, /align_center $
		, font = 'helvetica*14*bold' )

	; Transpose Button
	; self.tp = 'Transpose'
	!trifenestra.self -> widgets, widget_button( transformBase $
		, value= 'Transpose' $
		, uname='transpid' $
		, uvalue = 'TranspEvent' $
		, event_pro='trifenestra_event')

	; Q -> Angle Button
	; self.qorangle = 'Q -> Angle'
	;qtoangleid = widget_button( imagebase $
	;	, value='Q -> Angle' $
	;	, uname='qtoangle' $
	;	, uvalue = 'qtoangleEvent' $
	;	, event_pro='trifenestra_event')

	; I -> log(I) Button
	; self.log_scale = 'I -> log(I)'

	!trifenestra.self -> widgets, widget_button( transformBase $
		, value= 'I -> Log(I)' $
		, uname ='log_scale' $
		, uvalue = 'dologscaleEvent' $
		, event_pro = 'trifenestra_event' $
		)

	!trifenestra.self -> widgets, widget_button( transformBase $
		, value= 'Raw Button' $
		, uname ='rawButton' $
		, uvalue = 'rawButton' $
		, event_pro = 'trifenestra_event' $
		)

	!trifenestra.self -> widget_control, 'rawButton', sensitive = 0

	return, transformBase

end
;*******************************************************************************
;23456789 123456789 123456789 123465798 123456789 123456789 123465789 123456789
;       10        20        30        40        50        60        70        80
;*******************************************************************************
